@@ -6,8 +6,6 @@
 ;;;; Purpose:  Low-level ODBC API using UFFI
 ;;;; Authors:  Kevin M. Rosenberg and Paul Meurer
 ;;;;
-;;;; $Id$
-;;;;
 ;;;; This file, part of CLSQL, is Copyright (c) 2004 by Kevin M. Rosenberg
 ;;;; and Copyright (C) Paul Meurer 1999 - 2001. All rights reserved.
 ;;;;
@@ -26,10 +24,11 @@ May be locally bound to something else if a certain type is necessary.")
 (defvar *binary-format* :unsigned-byte-vector)
 (defvar *time-conversion-function*
     (lambda (universal-time &optional fraction)
-      (declare (ignore fraction))
-      (clsql-sys:format-time
-       nil (clsql-sys:utime->time universal-time)
-       :format :iso)
+       (let ((time (clsql-sys:utime->time universal-time)))
+	 (setf time (clsql-sys:time+
+		     time
+		     (clsql-sys:make-duration :usec (/ fraction 1000))))
+	 (clsql-sys:format-time nil time :format :iso))
       #+ignore
       universal-time)
    "Bound to a function that converts from a Lisp universal time fixnum (and a fractional
